/*
 *    DbReader.java
 *    Copyright (C) 1999 Pierre Geurts
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

import java.io.*;

public class DbReader {
  private static void skipBlankLines(StreamTokenizer stt) throws IOException {
    while (stt.ttype==stt.TT_EOL)
	stt.nextToken();
  }
  /* return 1 if there is an error, 0 otherwise */
  public static int loadDb (Database db, String filename) {
    boolean objectname=false;
    int objectnameindex=0;
    try {
      File f=new File(filename);
      Reader in=new FileReader(f);
      StreamTokenizer stt=new StreamTokenizer(in);
      /* some streamtokenizer initialisations*/
      stt.lowerCaseMode(true);
      stt.parseNumbers();
      stt.eolIsSignificant(true);
      stt.commentChar(';');
      /* file reading */
      stt.nextToken();
      skipBlankLines(stt);

      /*******************************
       * read attributes declaration *
       *******************************/
      int nreadvalue=0;
      int natt=0;
      while (stt.ttype!=stt.TT_EOL) {
	String attname;
	String atttype;
	if (stt.ttype!=stt.TT_WORD) {
	  System.out.println("error in attribute declaration");
	  return 1;
	}
	attname=stt.sval;
	stt.nextToken();
	if (stt.ttype!=stt.TT_WORD) {
	  System.out.println("error in attribute declaration");
	  return 1;
	}
	atttype=stt.sval;
	if (attname.equals("object")) {
	  if (!objectname) {
	    objectname=true;
	    objectnameindex=nreadvalue;
	  } else {
	    System.out.println("error in attribute declaration");
	    return 1;
	  }
	} else {
	  if (atttype.equals("symbolic"))
	    new SymbAtt(attname,db);
	  else if (atttype.equals("numerical")) 
	    new NumAtt(attname,db);
	  else {
	    System.out.println("not recognised attribute type");
	    return 1;
	  }
	  natt++;
	}
	nreadvalue++;
	stt.nextToken();
      }
      System.out.println("" + natt + " attributes have been read");
      System.out.println(db.attributes.toString());
      
      /**************************
       * read objects definitions *
       **************************/
      
      int exactnatt=natt;
      int nobject=1;
      while(stt.ttype!=stt.TT_EOF) {
	skipBlankLines(stt);
	/* number of readed values*/
	nreadvalue=0;
	/* number of readed attributes (as nreadvalue but 
	   without the object name) */
	natt=0;
	Obj o;
	if (objectname)
	  o=new Obj("",db);
	else
	  o=new Obj("O" + nobject,db);
	nobject++;
	
	/* read attributes values */
	while(stt.ttype!=stt.TT_EOL && stt.ttype!=stt.TT_EOF) {
	  if (objectname==true && 
	      objectnameindex==nreadvalue && 
	      nreadvalue==objectnameindex) {
	    if (stt.ttype==stt.TT_WORD) {
	      o.name=stt.sval;
	      nreadvalue++;
	    } else {
	      System.out.println("Object name must be a string");
	      return 1;
	    }
	  } else {
	    if (((Attribute)db.attributes.elementAt(natt)).ReadAttValue(o,stt)
		==1) {
	      return 1;
	    }
	    natt++;
	    nreadvalue++;
	  }
	  stt.nextToken();
	}
	if (natt<exactnatt) {
	  System.out.println("Attribute values are missing");
	  return 1;
	}
	skipBlankLines(stt);
      }
      System.out.println("" + nobject + " objects have been read");
      System.out.println(db.objects.toString());
      in.close();
    }
    catch (IOException e){
      System.err.println(e);
    }
    return 0;
  }
}
    
