/*
 *    DetailedNodeDrawer.java
 *    Copyright (C) 1999 Pierre Geurts
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

import java.awt.*;
import java.util.*;

/**
 *  a abstract class to allow many representation for
 *  a node
 */

public class DetailedNodeDrawer extends NodeDrawer {
  
  FontMetrics metrics_small,metrics_big, metrics_current;
  Font smallfont, bigfont;
  int SMALL_FIXED_FONT_HEIGHT=8;
  int BIG_FIXED_FONT_HEIGHT=14;
  int current_font_height;
  int default_border=10;
  Vector colours=new Vector();
  int max_width=60;
  int max_height=40;
  int max_ls_size=1;
  double coef;
  Canvas cv;

  DetailedNodeDrawer(Canvas c) {
    cv=c;
    bigfont=new Font("Helvetica", Font.PLAIN, BIG_FIXED_FONT_HEIGHT);
    c.setFont(bigfont);
    metrics_big=c.getFontMetrics(c.getFont());
    
    smallfont=new Font("Helvetica", Font.PLAIN, SMALL_FIXED_FONT_HEIGHT);
    c.setFont(smallfont);
    metrics_small=c.getFontMetrics(c.getFont());

    metrics_current=metrics_small;
    current_font_height=SMALL_FIXED_FONT_HEIGHT;
    
    coef=Math.exp(1)*20;
    colours.addElement(Color.black);
    colours.addElement(Color.red);
    colours.addElement(Color.green);
    colours.addElement(Color.blue);
    colours.addElement(Color.orange);
    colours.addElement(Color.darkGray);
    colours.addElement(Color.cyan);
    colours.addElement(Color.gray);
    colours.addElement(Color.magenta);
    colours.addElement(Color.pink);
    colours.addElement(Color.black);
    colours.addElement(Color.yellow);
    colours.addElement(Color.white);
    colours.addElement(Color.red);
    colours.addElement(Color.green);
    colours.addElement(Color.blue);
    colours.addElement(Color.orange);
    colours.addElement(Color.darkGray);
    colours.addElement(Color.cyan);
    colours.addElement(Color.gray);
    colours.addElement(Color.magenta);
    colours.addElement(Color.pink);
    colours.addElement(Color.black);
    colours.addElement(Color.yellow);
    colours.addElement(Color.white);
  }
  
  public void initWidth(WNode t) {
    t.width=0;
    if (t.sibling!=null) {
      initWidth(t.sibling);
    }
    if (t.child!=null) {
      initWidth(t.child);
    }
  }
  
  public void computeNodeSize(WNode t) {
    max_ls_size=t.dtnode.subset.size;
    computeSize(t);
  }
  
  public void computeSize(WNode t) {
    if (t.width==0) { //-- we compute the size only once
      double correction;
      if (t.dtnode.subset.size!=0) 
	correction=Math.log(coef)/Math.log(coef*max_ls_size/t.dtnode.subset.size);
      else
	correction=Math.log(coef)/Math.log(coef*max_ls_size);
      t.height=(int)(max_width*correction);
      t.width=(int)(max_height*correction);
    }
    
    t.border=default_border;
    
    if (t.sibling!=null) {
      computeSize(t.sibling);
    }
    
    if (t.child!=null) {
      computeSize(t.child);
    }
  }
  
  public void drawArc(Graphics g, int x1, int y1, int x2, 
		      int y2, String arclabel) {
    g.setColor(Color.black);
    g.drawLine((int)(x1*zoom), (int)(y1*zoom), (int)(x2*zoom),(int)(y2*zoom));
    g.drawString(arclabel,
		 (int)(((x1 + x2)*zoom - metrics_current.stringWidth(arclabel)) / 2),
		 (int)((y1 + y2)*zoom/2));
  }

  public void drawNode(Graphics g, WNode t) {
    int tposx=(int)(t.pos.x*zoom);
    int tposy=(int)(t.pos.y*zoom);
    int tw=(int)(t.width*zoom);
    int th=(int)(t.height*zoom);
    
    int current_x=tposy*t.dtnode.subset.size;
    int incr_x;
    int y=tposx;

    //-- draw the box
    if (t.dtnode.subset.size!=0) {
      for (int i=0; i<t.dtnode.subset.summary.length; i++) {
	if (t.dtnode.subset.summary[i]!=0) {
	  incr_x=th*t.dtnode.subset.summary[i];
	  g.setColor((Color)colours.elementAt(i+1));
	  g.fillRect((int)(current_x/t.dtnode.subset.size),y,
		     (int)(incr_x/t.dtnode.subset.size)+1,tw);
	  current_x+=incr_x;
	}
      }
    }

    g.setColor(Color.black);
    g.drawRect(tposy, tposx, th, tw);
    
    //-- we highlight the current open node
    if (t.selected) {
      g.setColor(Color.pink);
      g.drawRect(tposy-current_font_height, tposx-current_font_height, 
		 th+2*current_font_height, tw+2*current_font_height);
    }

    //-- we put information beneath the box ...
    String bottomtext;
    if (t.dtnode.type.equals("test"))
      bottomtext=t.label;
    else if (t.dtnode.type.equals("open")) 
      bottomtext="open";
    else if (t.dtnode.type.equals("open")) 
      bottomtext="leaf : " + t.dtnode.majclass;
    else
      bottomtext="error";
    
    g.setColor(Color.black);
    g.drawString(t.label, 
		 tposy + (th - metrics_current.stringWidth(t.label)) / 2,
		 tposx + tw + current_font_height);

    //-- ... and on top of the box
    g.drawString(t.dtnode.name, tposy, tposx+1);
    
  }
  
  // increase the zoom
  public void increaseZoom() {
    if (zoom==0.5)
      zoom=0.7;
    else if (zoom==0.7)
      zoom=1.0;
    else if (zoom==1.0)
      zoom=1.2;
    else if (zoom==1.2) {
      zoom=1.5;
      metrics_current=metrics_big;
      current_font_height=BIG_FIXED_FONT_HEIGHT;
      cv.setFont(bigfont);
    } else if (zoom==1.5)
      zoom=2.0;
    else if (zoom==2.0)
      zoom=2.5;
    else if (zoom==2.5)
      zoom=3.0;
  }
  
  // decrease the zoom
  public void decreaseZoom() {
    if (zoom==3.0)
      zoom=2.5;
    else if (zoom==2.5)
      zoom=2.0;
    else if (zoom==2.0)
      zoom=1.5;
    else if (zoom==1.5) {
      zoom=1.2;
      metrics_current=metrics_small;
      current_font_height=SMALL_FIXED_FONT_HEIGHT;
      cv.setFont(smallfont);
    } else if (zoom==1.2)
      zoom=1.0;
    else if (zoom==1.0)
      zoom=0.7;
    else if (zoom==0.7)
      zoom=0.5;
  }
}
