/*
 *    NumAtt.java
 *    Copyright (C) 1999 Pierre Geurts
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

import java.io.*;
import java.util.*;

public class NumAtt extends Attribute{
  /* le domaine de l'attribut dans la bd */
  float rangemin=(float)1.0, rangemax=(float)0.0;
  /* le meilleur score precedemment obtenu par cet attribut */
  float bestthreshold;

  NumAtt(String s, Database dab) {
    name=s;
    db=dab;
    index=db.addAttToDb(this);
  } 
  public void SetAttValue(Obj o,Object f) {
    float fl=((Float)f).floatValue();
    if (rangemin>rangemax) {
      rangemin=fl;
      rangemax=fl;
    } else {
      if (fl<rangemin)
	rangemin=fl;
      else if (fl>rangemax)
	rangemax=fl;
    }
    o.attributesvalues.setElementAt(f,index);
  }
  public int ReadAttValue(Obj o, StreamTokenizer stt) {
    if (stt.ttype==stt.TT_NUMBER) {
      SetAttValue(o,new Float(stt.nval));
      return 0;
    } else {
      System.out.println("A numerical value was expected");
      return 1;
    }
  }

  public String describe() {
    return toString()+ " - numerical - range : [" + rangemin + "," + rangemax + "]";
  }
  
  public String describeBestTest() {
    return toString() + " - threshold : " + bestthreshold + " - Information gain : " + 
      lastbestscore;
  }

/* return the best partition for a numerical attribute
 * we assume that the best threshold is already computed (by
 * using bestscore
 */
  public Partition bestpartition(ObjectSet os, SymbAtt goal) {
    TestNum t=new TestNum(this,bestthreshold);
    Vector v=os.splitOn(t);
    return new Partition(t,v);
  }

/* searches the best threshold value for a numerical attribute
 * assumes that the objectset size is greater than 0
 */
  public double bestscore(ObjectSet os, SymbAtt goal) {
    // first, the set is ordered by increasing attribute values 
    os.orderOn(this);

    // then prepare the data, we suppose that we have already
    // computed the entropy value
    int gvs=goal.values.size();
    int freqyes[]=new int[gvs+1];
    int freqno[]=new int[gvs+1];
    // in the beginning, the test is false for all objects
    for (int i=0; i<gvs; i++) {
      freqno[i]=os.summary[i];
    }
    freqno[gvs]=os.size;
    float currentthreshold=rangemin;
    double bestscore=0,actualscore=os.entropy;
    double entyes,entno;
    int ico;
    int iconext=goal.values.indexOf(goal.Value((Obj)os.elementAt(0)));
    float av;
    float avnext=((Float)Value((Obj)os.elementAt(0))).floatValue();

    for (int i=0; i<os.size-1; i++) {
      // index of the class for this object
      ico=iconext;
      iconext=goal.values.indexOf(goal.Value((Obj)os.elementAt(i+1)));
      av=avnext;
      avnext=((Float)Value((Obj)os.elementAt(i+1))).floatValue();
      freqno[ico]--;freqno[gvs]--;
      freqyes[ico]++;freqyes[gvs]++;
      if (
	  //	  ico!=iconext && // i'm not sure of this -> skip it
	  av!=avnext
	   ) { 
	// only if the class of this point is different from the next point
	// class and if its attribute value if also different from the next
	// point attribute value
	entno=Entropy.entropy(freqno,gvs,freqno[gvs]);
	entyes=Entropy.entropy(freqyes,gvs,freqyes[gvs]);
	actualscore=os.entropy-(freqno[gvs]*entno+freqyes[gvs]*entyes)/os.size;
	if (actualscore>bestscore) {
	  bestscore=actualscore;
	  currentthreshold=(float)((av+avnext)/2.0);
	}
      }
    }
    bestthreshold=currentthreshold;
    lastbestscore=bestscore;
    return bestscore;
  }
}
