import java.awt.*;

/**
 * "There flowered a White Tree, and that was for Gondor; but Seven Stars
 *  were about it, and a high crown above it, the signs of Elendil that no
 *  lord had borne for years beyond count.  And the stars flamed in the
 *  sunlight, for they were wrought of gems by Arwen daughter of Elrond;
 *  and the crown was bright in the morning, for it was wrought of mithril
 *  and gold."                    - J. R. R. Tolkien, _The Return of the King_
 *
 * Basic canvas for creating and displaying a WalTree.
 */
public class WalTreeCanvas extends Canvas {
  int default_border;

  WTFactory wt_factory;  // contain functions to draw a tree
  NodeDrawer nd;         // contain functions to draw a node
  WNode t;
  boolean dirty; // tell us if the node positions should be computed again
  int root_x;

  /**
   * Constructs new WalTreeCanvas, an associated WTFactory, and inits state.
   */
  public WalTreeCanvas() {
    super();

    wt_factory = new WTFactory();
    t = null;
    default_border = 5;
    root_x = 8;
    dirty = false;
  }

  public Dimension getPreferredSize() {
    return new Dimension(2000, 2000);
  }

  public Dimension getMiminimumSize() {
    return new Dimension(40, 40);
  }

  /**
   * Set the tree to be displayed by this canvas.
   */
  public void setTree(WNode t) {
    this.t = t;
    dirty = true;
  }

  /**
   * Set the default border size for nodes 
   */
  public void setDefaultBorder(int b) {
    default_border = b;
  }

  /**
   * Set the distance between parent nodes.
   */
  public void setParentDistance(int val) {
    wt_factory.setParentDistance(val);
  }

  /**
   * Set the horizontal offset between edge of canvas and root node.
   */
  public void setRootOffset(int val) {
    root_x = val;
  }

  /**
   * Draw the tree associated with this canvas.
   */
  public void paint(Graphics g) {
    super.paint(g);

    Dimension d = getSize();

    /* Wipe background */
    g.setColor(Color.white);
    g.fillRect(0, 0, d.width, d.height);

    if(t == null) {
      return;
    }

    if(dirty == true) {
      /* Calculate node offsets */
      wt_factory.layout(t); 

      /* Calculate absolute node positions */

      wt_factory.plantTree(t, root_x, root_x-wt_factory.computeMaxOffset(t,0));

      dirty = false;
    }

    /* Paint the beastie */
    wt_factory.paintFullTree(g, t, nd);
  }
};
